% Setups environment
clear;
data = load('faces.mat');

tr = data.tr;
te = data.te;
w = data.w;
h = data.h;

%% Fraction of hot faces
disp(['Fraction of hot faces: ', num2str(mean(tr.hot))]);

%% Look at some faces
figure(1);
subplot(1,2,1);
imagesc( reshape(mean(tr.images(not(tr.hot),:)), h, w) );
title('Not Hot');
colormap('gray');
axis('off');

subplot(1,2,2);
imagesc( reshape(mean(tr.images(tr.hot,:)), h, w) );
title('Hot');
colormap('gray');
axis('off');

%% Plot the distribution of the off diagonal elements
sigma = cov(tr.images);
tmp = sigma(eye(size(sigma)) == 0);
figure(2);
subplot(1,2,1);
hist(tmp(:), 100);
title('Off-Diagonal Terms');
xlabel('Covariance Values');
ylabel('Frequency');

subplot(1,2,2);
hist(diag(sigma), 100);
title('Diagonal Terms (Variance)');
xlabel('Variance Values');
ylabel('Frequency');


%% Compute the top 100 eigenfaces
[efaces, xbar, svalues] = eigenfaces(tr.images, 100);

%% Plot the top 10 eigen faces
figure(3);
for i = 1:10
    subplot(2,5,i);
    imagesc(reshape(efaces(i,:), h, w));
    colormap('gray');
    axis('off');
end

%% Plot the Singular values
figure(4);
plot(svalues)
title('Singular Values Plot');

%% subspace cov
[z, xapprox100, mse] = projection(tr.images, efaces, xbar, 100);

sigma = cov(z);
tmp = sigma(eye(size(sigma)) == 0);
figure(5);
subplot(1,2,1);
hist(tmp(:), 100);
title('Off-Diagonal Terms');
xlabel('Covariance Values');
ylabel('Frequency');

subplot(1,2,2);
hist(diag(sigma), 100);
title('Diagonal Terms (Variance)');
xlabel('Variance Values');
ylabel('Frequency');


%% Project the images into the 10 dimensional eigenspace
[z, xapprox10, mse] = projection(tr.images, efaces, xbar, 10);
[z, xapprox50, mse] = projection(tr.images, efaces, xbar, 50);



%% Plot before and after images
figure(6);
for i = 1:10
   subplot(4, 10, i);
   imagesc(reshape(tr.images(i,:), h, w));
   colormap('gray');
   axis('off');
   
   subplot(4, 10, i + 10);
   imagesc(reshape(xapprox10(i,:), h, w));
   colormap('gray');
   axis('off');
   
   subplot(4, 10, i + 20);
   imagesc(reshape(xapprox50(i,:), h, w));
   colormap('gray');
   axis('off');
   
   subplot(4, 10, i + 30);
   imagesc(reshape(xapprox100(i,:), h, w));
   colormap('gray');
   axis('off');
end

%% Train / Test with different numbers of principal components
split = round(4/5 * tr.n);
trials = 100;
accuracy = zeros(100, trials);
for p = 1:100
    % Compute the projection
    [X, xapprox, mse] = projection(tr.images, efaces, xbar, p);
    y = tr.hot;
    for t = 1:trials
        % Do the random split
        ind = randperm(tr.n);
        Xtr = X(ind(1:split), :);
        ytr = y(ind(1:split));
        Xval = X(ind((split+1):end),:);
        yval = y(ind((split+1):end));
        % Train the Gaussian naive bayes classifier
        bn = train_gnb(Xtr, ytr);
        % Predict the y values for the heldout data
        ypred = pred_gnb(bn, Xval);
        % Compute the l1 error
        accuracy(p,t) = mean( yval == round(ypred) );
    end
end

%% Examine the accuracy in the predictions
figure(7);
[optacc, popt] = max(mean(accuracy,2));
plot(mean(accuracy,2));
title(['Optimal # Components popt = ' num2str(popt)]);
xlabel('Number of components');
ylabel('Prediction Accuracy');


%% Compute the prediction accuracy of the test data
[Xtr, xapprox, mse] = projection(tr.images, efaces, xbar, popt);
[Xte, xapprox, mse] = projection(te.images, efaces, xbar, popt);

bn = train_gnb(Xtr, tr.hot);
pred = pred_gnb(bn, Xte);
disp(['Background; ', num2str(max(mean(te.hot), mean(not(te.hot))))]);
disp(['Accuracy: ', num2str( mean( te.hot == round(pred) ) ) ]);























